program Normaliser1;

uses
  Forms, SysUtils,
  Voxel in 'Voxel.pas',
  Voxel_Tools in 'Voxel_Tools.pas';

{$R *.RES}

var
   SrcFilename,
   DestFilename: string;
   src: TVoxel;

procedure PrintFacingReport(section: TVoxelSection);
var report: FacingReport;
    outfile: Text;
    normals_scheme: string;
begin
     // prepare section
     WriteLn('Analysing section "',Trim(section.Name),'"..');
     WriteLn(removeRedundantVoxels(section),' redundant voxels removed');
     WriteLn('Calculating facings now..');
     // open outfile
     AssignFile(outfile,DestFilename);
     if FileExists(DestFilename) then
        Append(outfile)
     else
         Rewrite(outfile);
     // generate report
     CalcFacings(section);
     report := GenerateFacingReport(section);
     WriteLn(outfile,'Normals Analysis of "',SrcFilename,'!',Trim(Section.Name),'"..');
     case Section.Tailer.Unknown of
          2: normals_scheme := 'TIBERIAN_SUN';
          4: normals_scheme := 'RED_ALERT2';
          else normals_scheme := 'UNKNOWN';
     end;
     WriteLn(outfile,'Normals scheme = ',Section.Tailer.Unknown,' ',normals_scheme);
     voxel_tools.PrintFacingReport(report,outfile);
     WriteLn(outfile);
     // close outfile
     CloseFile(outfile);
end;

function DetermineFiles: boolean; // false if filenames not found (app should exit)
type errors = (NONE, BAD_PARAMS, SRC_NOT_EXIST, DEST_EXIST);
var error: errors;
begin
     error := NONE;
     // determine files
     if (ParamCount <> 2) then
        error := BAD_PARAMS
     else begin
         SrcFilename := ParamStr(1);
         if not FileExists(SrcFilename) then
            error := SRC_NOT_EXIST
         else begin
              DestFilename := ParamStr(2);
              if FileExists(DestFilename) then
                 error := DEST_EXIST;
         end;
     end;
     // user message?
     if error <> NONE then begin
        Result := False; // assume bad
        case error of
             BAD_PARAMS: WriteLn('Usage: ',ExtractFileName(ParamStr(0)),' [source vxl filename] [dest txt report filename]');
             SRC_NOT_EXIST: WriteLn('Error: the source file "',SrcFilename,'" does not exist!');
             DEST_EXIST: WriteLn('Error: the destination file "',DestFilename,'" exists!');
        end;
     end else
         Result := True;
end;

function LoadSrc(srcFilename: string): boolean; // return success
var s: integer;
begin
     // load voxel file
     WriteLn('Loading "',srcFilename,'"..');
     src := TVoxel.Create;
     src.LoadFromFile(srcFilename);
     if src.ErrorCode <> OK then begin
        WriteLn('Error loading source voxel file: ',src.ErrorMsg);
        Result := False;
        Exit;
     end;
     // list sections for info
     if Length(src.Section) = 0 then begin
        WriteLn('Error: Voxel file contains no sections; nothing to do!');
        Result := False;
        Exit;
     end;
     WriteLn('There are ',Length(src.Section),' section(s):');
     for s := 0 to Length(src.Section)-1 do
         with src.Section[s] do
              WriteLn(Header.Number,' ',Name,' (',Tailer.XSize,'x',Tailer.YSize,'x',Tailer.ZSize,')');
     // done
     Result := True;
end;

var sec: Integer;
begin
  // not sure we need this Application object?
  Application.Initialize;
  Application.Run;
  // preamble
  WriteLn('Will''s patent Voxel Normals Analyser v1, Nov 2001');
  // parse the source and destination files
  if not DetermineFiles then
     Halt(1);
  // load the source file
  if not LoadSrc(srcFilename) then
     Halt(2);
  // do the analysis
  for sec := 0 to Length(src.Section)-1 do
      PrintFacingReport(src.Section[sec]);
  // done
  WriteLn('Report written to "',DestFilename,'"');
  //ReadLn;
end.
